Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.ServiceModel
Imports Microsoft.ServiceModel.Samples.Syndication

    '* This class provides an implementation for the ItemListService service, 
    '* which provides RSS and Atom feeds from one service implementation  
    '* Modify this class to create your own RSS/Atom feed implementation.
    '* Note: To implement multiple ServiceContracts on the same service, use interfaces 
    '* to define those contracts then implement the interfaces on the service class

    <ServiceContract()> _
    Class ItemListService

        'GetItemList returns a content feed 
        <OperationContract()> _
        Public Function GetItemList() As ContentFeed(Of FeedItem)

            'You could also use DataTableContentFeed if your data resides in a database
            'see the documentation for ways to generate and extend content feeds
            Dim feed As ContentFeed(Of FeedItem) = New ContentFeed(Of FeedItem)()
            feed.FeedAuthor = "wcfuser@contoso.com"
            feed.FeedTitle = "WCF Item List!"
            feed.FeedDescription = "A sample showing how to use FeedItem directly"
            feed.FeedId = "http://contoso.com/itemlist"
            feed.FeedWebPageLink = "http://www.contoso.com/itemlist"

            'populate the list of feed items
            feed.FeedItems = GenerateItemList()

            Return feed
        End Function

        ' GenerateItemList manually generates a number of sample feed items
        Protected Function GenerateItemList() As ItemList(Of FeedItem)

            ' Create new list of RSS feed items
            Dim list As ItemList(Of FeedItem) = New ItemList(Of FeedItem)()

            ' Create RSS feed item
            Dim item1 As FeedItem = New FeedItem()
            item1.ItemTitle = "This is item 1"

            ' Create content for feed item 
            ' Treat the content of this item as a string and escape it
            item1.ItemContent = New ItemContent( _
                "<div>This escaped string item is <i>quite</i> <span style='color:red'> hot! </span><br/>Many readers " _
                & "will interpret this as HTML even when it's escaped. Escaped strings are your best bet for HTML " _
                & "content in both RSS and Atom.</div>", False)
            item1.ItemPublishDate = DateTime.Parse("6/1/2006")
            item1.ItemLastUpdated = item1.ItemPublishDate.Value.AddDays(1)
            item1.ItemCategories.Add(New FeedCategory("http://contoso.com/categories/", "Everyone"))
            item1.ItemExtensions.Add(New ObjectExtension("Status", "InProgress", "http://contoso.com/itemlist"))
            item1.ItemId = "http://contoso.com/feed/items/7333830f-a644-430f-8476-ce697d27d38e"

            'add item1 to the list
            list.Add(item1)

            ' ... add more items to the list here 

            ' Return list of feed items
            Return list
        End Function
    End Class
