//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.Reflection;
    using System.Runtime.Serialization;
    using System.ServiceModel;
    using System.ServiceModel.Description;
    using System.ServiceModel.Dispatcher;

    public class DataContractFormatterBehavior : IEndpointBehavior
    {
        public void AddBindingParameters(ServiceEndpoint serviceEndpoint, System.ServiceModel.Channels.BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyClientBehavior(ServiceEndpoint serviceEndpoint, ClientRuntime behavior)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "DataContractFormatterBehavior.ApplyClientBehavior");
            IOperationBehavior opbehavior;
            foreach (OperationDescription operationDescription in serviceEndpoint.Contract.Operations)
            {
                opbehavior=new DataContractSerializerOperationBehavior(operationDescription);
                operationDescription.Behaviors.Remove<DataContractSerializerOperationBehavior>();
                operationDescription.Behaviors.Remove<XmlSerializerOperationBehavior>();
                operationDescription.Behaviors.Add(opbehavior);
                foreach (ClientOperation operationRuntime in behavior.Operations)
                {
                    if (operationDescription.Name == operationRuntime.Name)
                    {
                        opbehavior.ApplyClientBehavior(operationDescription,operationRuntime);
                    }
                }
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully applied DataContractFormatter", "DataContractFormatterBehavior.ApplyClientBehavior");
            }
        }

        public void ApplyDispatchBehavior(ServiceEndpoint serviceEndpoint, EndpointDispatcher endpointDispatcher)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "DataContractFormatterBehavior.ApplyDispatchBehavior");
            IOperationBehavior opbehavior;
            foreach (OperationDescription operationDescription in serviceEndpoint.Contract.Operations)
            {
                opbehavior = new DataContractSerializerOperationBehavior(operationDescription);
                operationDescription.Behaviors.Remove<DataContractSerializerOperationBehavior>();
                operationDescription.Behaviors.Remove<XmlSerializerOperationBehavior>();
                operationDescription.Behaviors.Add(opbehavior);
                foreach (DispatchOperation operationRuntime in endpointDispatcher.DispatchRuntime.Operations)
                {
                    if (operationDescription.Name == operationRuntime.Name)
                    {
                        opbehavior.ApplyDispatchBehavior(operationDescription, operationRuntime);
                    }
                }
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Successfully applied DataContractFormatter", "DataContractFormatterBehavior.ApplyDispatchBehavior");
            }
        }

        public void Validate(ServiceEndpoint serviceEndpoint)
        {
            
        }
    }
}
