//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.ServiceModel;
    using System.ServiceModel.Dispatcher;
    using System.ServiceModel.Description;
    using System.ServiceModel.Channels;

    public class UriDispatchBehavior : IEndpointBehavior
    {
        public void AddBindingParameters(ServiceEndpoint serviceEndpoint, System.ServiceModel.Channels.BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyClientBehavior(ServiceEndpoint serviceEndpoint, ClientRuntime behavior)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "UriOperationSelectorBehavior.ApplyClientBehavior");
            Dictionary<string, string> actionDictionary = new Dictionary<string, string>();
            foreach (OperationDescription operation in serviceEndpoint.Contract.Operations)
                actionDictionary.Add(operation.Messages[0].Action, operation.Name);

            behavior.ManualAddressing = false;
            behavior.MessageInspectors.Add(new UriPathSuffixOperationAddresser(actionDictionary));
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Added UriPathSuffixOperationAddresser to runtime", "UriOperationSelectorBehavior.ApplyClientBehavior");
        }

        public void ApplyDispatchBehavior(ServiceEndpoint serviceEndpoint, EndpointDispatcher endpointDispatcher)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "UriOperationSelectorBehavior.ApplyDispatchBehavior");
            if (serviceEndpoint == null)
                throw new ArgumentNullException("serviceEndpoint");

            if (endpointDispatcher == null)
                throw new ArgumentNullException("endpointDispatcher");

            Dictionary<string, string> operationNameDictionary = new Dictionary<string, string>();
            foreach (OperationDescription operation in serviceEndpoint.Contract.Operations)
            {
                try
                {
                    operationNameDictionary.Add(operation.Name.ToLower(), operation.Name);
                }
                catch (ArgumentException)
                {
                    Exception ex= new Exception(String.Format("The specified contract cannot be used with case insensitive URI dispatch because there is more than one operation named {0}", operation.Name));
                    TraceHelper.WriteTraceData(System.Diagnostics.TraceEventType.Error, 0, "Duplicate operation names", ex.Message, "UriOperationSelectorBehavior.ApplyDispatchBehavior", ex);
                    throw ex;
                }
            }
            endpointDispatcher.AddressFilter = new PrefixAddressFilter(serviceEndpoint.ListenUri);
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Endpoint's AddressFilter is now PrefixAddressFilter", "UriOperationSelectorBehavior.ApplyDispatchBehavior");
            endpointDispatcher.ContractFilter = new MatchAllMessageFilter();
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Endpoint's ContractFilter is now MatchAllMessageFilter", "UriOperationSelectorBehavior.ApplyDispatchBehavior");
            endpointDispatcher.DispatchRuntime.OperationSelector = new UriPathSuffixOperationSelector(serviceEndpoint.Address.Uri, operationNameDictionary);
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, "Added UriPathSuffixOperationSelector to runtime", "UriOperationSelectorBehavior.ApplyDispatchBehavior");

        }

        public void Validate(ServiceEndpoint serviceEndpoint)
        {
            if (serviceEndpoint == null)
                throw new ArgumentNullException("serviceEndpoint");

        }
    }
}