using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using System.ServiceModel;
using System.ServiceModel.Configuration;
using Microsoft.ServiceModel.Samples.Syndication;

namespace Microsoft.ServiceModel.Samples.Syndication.Configuration
{
    public class MessageTransformBehaviorSection : BehaviorExtensionElement
    {
        public  MessageTransformBehaviorSection() : base()
        {
            //default
            this.TransformProvider = typeof(Rss20TransformProvider).ToString();
        }
        protected override object CreateBehavior()
        {
            Type transformProvider = Type.GetType(this.TransformProvider);
            return new MessageTransformBehavior(transformProvider);
        }
        public override Type BehaviorType
        {
            get { return typeof(MessageTransformBehavior); }
        }

        [ConfigurationProperty("transformProvider", IsRequired = true)]
        public string TransformProvider
        {
            get { return (string)base["transformProvider"]; }
            set
            {
                ValidateType(value);
                base["transformProvider"] = value;
            }
        }
        public override void CopyFrom(ServiceModelExtensionElement from)
        {
            if (!(from is MessageTransformBehaviorSection))
            {
                throw new ArgumentException("from must be a MessageTransformBehavior Section");
            }
            MessageTransformBehaviorSection messagetransform = from as MessageTransformBehaviorSection;
            this.TransformProvider = messagetransform.TransformProvider;
        }
        void ValidateType(string typename)
        {

            
            Exception innerex=null;
            try
            {
                Type type = Type.GetType(typename);
                object instance = Activator.CreateInstance(type);
                if (instance != null)
                {
                    IFormatTransformProvider provider = instance as IFormatTransformProvider;
                    if (provider == null)
                    {
                        innerex = new ArgumentException("transformProvider type must implement IFormatTransformProvider");
                    }
                }
            }
            catch (Exception e)
            {
                innerex = e;
            }
            if (innerex != null)
            {
                throw new ArgumentException(String.Format("Specified transformProvider type [{0}] is not valid. See inner exception for more details",typename), innerex);
            }
        }
    }
}
