using System.Runtime.Serialization;
using System.Collections;
using System.Collections.Generic;
using System;
using System.Data;
using System.ServiceModel;
using System.Xml.Serialization;
namespace Microsoft.ServiceModel.Samples.Syndication
{
   [DataContract(
        Name = FeedConstants.FeedName,
        Namespace = FeedConstants.FeedNamespace)]
    [XmlType(
        TypeName= FeedConstants.FeedName,
        Namespace=FeedConstants.FeedNamespace)]
    public class ContentFeed<TItem> 
       where TItem : class 
    {
        private string titleField;
        private string webPagelinkField;
        private string descriptionField;
        private string languageField;
        private string authorField;
        private string copyrightField;
        private Nullable<DateTime> lastUpdatedField;
        private string imageField;
        private ItemList<TItem> itemField;
        private string generatorField;
        private List<FeedCategory> categoriesField;
        private string IdField;
        private string feedlinkField;
        private ContentExtensions extensions;
        void SetDefaults()
        {
            this.FeedItems = new ItemList<TItem>();
            this.FeedExtensions = new ContentExtensions();
            this.FeedGenerator = "Microsoft RSS Toolkit v0.5";
            this.FeedLanguage = System.Globalization.CultureInfo.CurrentCulture.IetfLanguageTag;
            this.FeedLastUpdated = DateTime.UtcNow;

            //atom requires <link rel='self> so try to get it from request address
            if (OperationContext.Current != null && 
                OperationContext.Current.RequestContext != null && 
                OperationContext.Current.RequestContext.RequestMessage != null && 
                OperationContext.Current.RequestContext.RequestMessage.Headers != null)            
            {
                if (OperationContext.Current.RequestContext.RequestMessage.Headers.To != null)
                {
                    this.FeedLink = OperationContext.Current.RequestContext.RequestMessage.Headers.To.ToString();
                }
                else if (OperationContext.Current.RequestContext.RequestMessage.Properties.ContainsKey("Via"))
                {
                    Uri via = OperationContext.Current.RequestContext.RequestMessage.Properties["Via"] as Uri;
                    if (via != null)
                    {
                        this.FeedLink = via.ToString();
                    }
                }
            }
        }
        public ContentFeed()
        {
            SetDefaults();
        }

        [DataMember(Name=FeedConstants.FeedId,Order=2)]
        [XmlElement(ElementName = FeedConstants.FeedId, Order = 2)]
        public virtual string FeedId
        {
            get
            {
                return this.IdField;
            }
            set
            {
                this.IdField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedTitle, Order = 0)]
        [XmlElement(ElementName = FeedConstants.FeedTitle, Order = 0)]
        public virtual string FeedTitle
        {
            get
            {
                return this.titleField;
            }
            set
            {
                this.titleField = value;
            }
        }

        [DataMember(Name = FeedConstants.FeedWebPageLink, Order = 3)]
        [XmlElement(ElementName = FeedConstants.FeedWebPageLink, Order = 3)]
        public virtual string FeedWebPageLink
        {
            get
            {
                return this.webPagelinkField;
            }
            set
            {
                this.webPagelinkField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedLink, Order = 4)]
        [XmlElement(ElementName = FeedConstants.FeedLink, Order = 4)]
        public virtual string FeedLink
        {
            get
            {
                return this.feedlinkField;
            }
            set
            {
                this.feedlinkField = value;
            }
        }

        [DataMember(Name = FeedConstants.FeedDescription, Order = 1)]
        [XmlElement(ElementName = FeedConstants.FeedDescription, Order = 1)]
        public virtual string FeedDescription
        {
            get
            {
                return this.descriptionField;
            }
            set
            {
                this.descriptionField = value;
            }
        }

        [DataMember(Name = FeedConstants.FeedLanguage, Order = 5)]
        [XmlElement(ElementName = FeedConstants.FeedLanguage, Order = 5)]
        public virtual string FeedLanguage
        {
            get
            {
                return this.languageField;
            }
            set
            {
                this.languageField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedCopyright, Order = 11)]
        [XmlElement(ElementName = FeedConstants.FeedCopyright, Order = 11)]
        public virtual string FeedCopyright
        {
            get
            {
                return this.copyrightField;
            }
            set
            {
                this.copyrightField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedLastUpdated, Order = 7)]
        [XmlElement(ElementName = FeedConstants.FeedLastUpdated, Order = 7)]
        public virtual Nullable<DateTime> FeedLastUpdated
        {
            get
            {
                return this.lastUpdatedField;
            }
            set
            {
                this.lastUpdatedField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedAuthor, Order = 8)]
        [XmlElement(ElementName = FeedConstants.FeedAuthor, Order = 8)]
        public virtual string FeedAuthor
        {
            get
            {
                return this.authorField;
            }
            set
            {
                this.authorField = value;
            }
        }

        [DataMember(Name = FeedConstants.FeedImage, Order = 9)]
        [XmlElement(ElementName = FeedConstants.FeedImage, Order = 9)]
        public virtual string FeedImage
        {
            get
            {
                return this.imageField;
            }
            set
            {
                this.imageField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedCategories, Order = 10)]
        [XmlElement(ElementName = FeedConstants.FeedCategories, Order = 10)]
        public virtual List<FeedCategory> FeedCategories
        {
            get
            {
                return this.categoriesField;
            }
            set
            {
                this.categoriesField = value;
            }

        }
        [DataMember(Name = FeedConstants.FeedGenerator, Order = 6)]
        [XmlElement(ElementName = FeedConstants.FeedGenerator, Order = 6)]
        public virtual string FeedGenerator
        {
            get
            {
                return this.generatorField;
            }
            set
            {
                this.generatorField = value;
            }
        }
        [DataMember(Name = FeedConstants.FeedExtensions, Order = 12)]
        [XmlElement(ElementName = FeedConstants.FeedExtensions, Order = 12)]
        public virtual ContentExtensions FeedExtensions
        {
            get
            {
                return this.extensions;
            }
            set
            {
                this.extensions = value;
            }

        }

        [DataMember(Name = FeedConstants.FeedItems, Order = 13)]
        [XmlElement(ElementName = FeedConstants.FeedItem, Order = 13)]
        public virtual ItemList<TItem> FeedItems
        {
            get
            {
                return this.itemField;
            }
            set
            {
                this.itemField = value;
            }
        }
    }

    
    [CollectionDataContract(
        ItemName=FeedConstants.FeedItem,
        Namespace=FeedConstants.FeedNamespace,
       Name = FeedConstants.FeedItemList), KnownType(typeof(FeedItem))]
    public class ItemList<T> : List<T>
    {
        public ItemList()
            : base()
        {
        }
    }
}
