using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using System.Xml;
using System.Data;
using System.ServiceModel.Channels;
using System.ServiceModel;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    [DataContract(Name = FeedConstants.DataRowFeedItem, Namespace = FeedConstants.FeedNamespace)]
    public class DataRowFeedItem : FeedItem
    {

        public DataRowFeedItem(DataRow row,string extensionNs, bool unescapedContent) : base()
        {
            this.ItemExtensions = new ContentExtensions();
            DataTable table = row.Table;
            foreach (DataColumn column in table.Columns)
            {
                if (row[column.Ordinal]==null || String.IsNullOrEmpty(row[column.Ordinal].ToString()))
                {
                    //skip null columns
                    continue;
                }
                if (column.ColumnName.StartsWith("Category_") && column.ColumnName.Length > "Category_".Length)
                {
                    this.ItemCategories.Add(new FeedCategory(column.ColumnName.Substring("Category_".Length), row[column.Ordinal].ToString()));
                }
                else
                {
                    switch (column.ColumnName)
                    {
                        case FeedConstants.ItemTitle:
                            this.ItemTitle = row[column.Ordinal].ToString();
                            break;
                        case FeedConstants.ItemContent:
                              this.ItemContent = new ItemContent(row[column.Ordinal].ToString(), unescapedContent);
                            break;
                        case FeedConstants.ItemPublishDate:
                            DateTime pub;
                            if (DateTime.TryParse(row[column.Ordinal].ToString(), out pub))
                            {
                                this.ItemPublishDate = pub;
                            }
                            break;
                        case FeedConstants.ItemLastUpdated:
                            DateTime lastup;
                            if(DateTime.TryParse(row[column.Ordinal].ToString(),out lastup))
                            {
                                this.ItemLastUpdated=lastup;
                            }
                            break;
                        case FeedConstants.ItemWebPageLink:
                            this.ItemWebPageLink = row[column.Ordinal].ToString();
                            break;
                        case FeedConstants.ItemId:
                            this.ItemId = row[column.Ordinal].ToString();
                            break;
                        case FeedConstants.ItemAuthor:
                            this.ItemAuthor = row[column.Ordinal].ToString();
                            break;
                        case FeedConstants.ItemCommentsLink:
                            this.ItemCommentsLink = row[column.Ordinal].ToString();
                            break;
                        default:
                            //just add it to extensions
                            ObjectExtension extension;
                            if (String.IsNullOrEmpty(column.Namespace))
                            {
                                extension = new ObjectExtension(column.ColumnName, row[column.Ordinal], extensionNs);
                            }
                            else
                            {
                                extension = new ObjectExtension(column, row[column.Ordinal]);
                            }

                            this.ItemExtensions.Add(extension);
                            break;
                    }
                }
            }
        }
        

    }
}
