using System.Xml.Serialization;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Collections;
using System.Collections.Generic;
using System;
using System.Data;
using System.ComponentModel;
using System.Reflection;

namespace Microsoft.ServiceModel.Samples.Syndication
{
    [DataContract(
        Name = FeedConstants.DataTableFeedName,
        Namespace = FeedConstants.FeedNamespace)]
    public class DataTableContentFeed : ContentFeed<DataRowFeedItem>
    {


        public DataTableContentFeed(DataTable table)
            : this(table, "http://tempuri.org/")
        {
        }

        public DataTableContentFeed(DataTable table, string extensionNs)
            : this(table,extensionNs,false)
        {

        }
        public DataTableContentFeed(DataTable table, string extensionNs, bool unescapedContent)
            : base()
        {
            if (table == null)
            {
                throw new ArgumentNullException("table");
            }
            if (extensionNs == null)
            {
                throw new ArgumentNullException("extensionNs");
            }
            this.FeedTitle = table.TableName;
            //get extended properties if they exist

            this.FeedDescription = GetExtendedProperty(table, FeedConstants.FeedDescription);
            this.FeedWebPageLink = GetExtendedProperty(table, FeedConstants.FeedWebPageLink);
            this.FeedAuthor = GetExtendedProperty(table, FeedConstants.FeedAuthor);
            this.FeedImage = GetExtendedProperty(table, FeedConstants.FeedImage);
            this.FeedCopyright = GetExtendedProperty(table, FeedConstants.FeedCopyright);
            this.FeedId = GetExtendedProperty(table, FeedConstants.FeedId);
            this.FeedLanguage = GetExtendedProperty(table, FeedConstants.FeedLanguage);
            this.FeedLink = GetExtendedProperty(table, FeedConstants.FeedLink);
            string link = GetExtendedProperty(table, FeedConstants.FeedLink);
            DateTime updated;
            if (DateTime.TryParse(GetExtendedProperty(table, FeedConstants.FeedLastUpdated), out updated))
            {
                this.FeedLastUpdated = updated;
            }
            if (!String.IsNullOrEmpty(link))
            {
                this.FeedLink = link;
            }
            
            foreach (DataRow row in table.Rows)
            {
                try
                {
                    DataRowFeedItem item = new DataRowFeedItem(row, extensionNs, unescapedContent);
                    this.FeedItems.Add(item);
                    if ((item.ItemLastUpdated != null) &&
                        (this.FeedLastUpdated == null || (item.ItemLastUpdated.Value > this.FeedLastUpdated.Value)))
                    {
                        this.FeedLastUpdated = item.ItemLastUpdated.Value;
                    }
                }
                catch (NullReferenceException ex)
                {
                    // TO DO trace it
                }
                catch (FormatException fex)
                {
                    // TO DO trace it
                }
            }

        }

        string GetExtendedProperty(DataTable table, string property)
        {
            if (table.ExtendedProperties.ContainsKey(property))
            {
                return table.ExtendedProperties[property].ToString();
            }
            else
            {
                return null;
            }
        }
    }
}