using System;
using System.Data;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Schema;
using System.Xml.Serialization;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class ObjectExtension : IXmlSerializable
    {
        private string nameField;
        private string namespaceField;
        private object valueField;
        static Dictionary<ExtensionKey, XmlSerializer> cachedSerializers = new Dictionary<ExtensionKey, XmlSerializer>(new ExtensionKeyComparer());
        static object lockobject = new object();

        public ObjectExtension(DataColumn column, object value)
            : this(column.ColumnName,value, column.Namespace)
        {
            if (column == null)
            {
                throw new ArgumentNullException("column");
            }
        }
        public ObjectExtension(string name, object value)
            : this(name, value, String.Empty)
        {
        }

        public ObjectExtension(string name, object value, string ns)
        {
            this.namespaceField = ns;
            this.nameField = name;
            this.valueField = value;
        }
        public string Name
        {
            get { return nameField; }

        }
        public string Namespace
        {
            get { return namespaceField; }

        }
        public object Value
        {
            get { return valueField; }
        }
        internal ExtensionKey GetKey()
        {
            ExtensionKey key=new ExtensionKey();
            key.ExtensionType = this.Value.GetType();
            key.Name = this.Name;
            key.Namespace = this.Namespace;
            return key;
        }
        internal static Dictionary<ExtensionKey, XmlSerializer> CachedSerializer
        {
            get { return ObjectExtension.cachedSerializers; }
        }
        #region IXmlSerializable Members

        public XmlSchema GetSchema()
        {
            return null;
        }

        public void ReadXml(XmlReader reader)
        {
            throw new Exception("The method or operation is not implemented.");
        }
        public void WriteXml(XmlWriter writer)
        {
            if (writer == null)
            {
                throw new ArgumentNullException("writer");
            }
            XmlSerializer serializer;
            ExtensionKey key = this.GetKey();
            if (ObjectExtension.CachedSerializer.ContainsKey(key))
            {
                serializer = ObjectExtension.CachedSerializer[key];
            }
            else
            {
                lock (lockobject)
                {
                    if (!ObjectExtension.cachedSerializers.ContainsKey(key))
                    {
                        XmlRootAttribute root = new XmlRootAttribute(this.Name);
                        root.Namespace = this.Namespace;
                        serializer = new XmlSerializer(this.Value.GetType(), root);
                        ObjectExtension.cachedSerializers.Add(key, serializer);
                    }
                    else
                    {
                        serializer = ObjectExtension.cachedSerializers[key];
                    }
                }
            }
            serializer.Serialize(writer, this.Value);
        }
        #endregion
        }


    internal class ExtensionKey
    {
        public override bool Equals(object obj)
        {
            ExtensionKey key = obj as ExtensionKey;
            return ((key != null) &&
                     (key.Name == this.Name) &&
                     (key.Namespace == this.Namespace) &&
                     (key.ExtensionType == this.ExtensionType));
        }
        public override int GetHashCode()
        {
            string s = String.Format("{0}-{1}-{2}", this.Namespace, this.Name, this.ExtensionType);
            return s.GetHashCode();
        }
        internal string Name;
        internal string Namespace;
        internal Type ExtensionType;

    }
    internal class ExtensionKeyComparer : IEqualityComparer<ExtensionKey>
    {
        #region IEqualityComparer<ExtensionKey> Members

        public bool Equals(ExtensionKey x, ExtensionKey y)
        {
            return x.Equals(y);
        }

        public int GetHashCode(ExtensionKey obj)
        {
            return obj.GetHashCode();
        }

        #endregion
    }
}
