using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;
using System.Diagnostics;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class ContentFeedHost : ServiceHost
    {

        static string RssContractNamespace = "http://schemas.microsoft.com/2006/04/serviceModel/feed/rss";
        static string AtomContractNamespace = "http://schemas.microsoft.com/2006/04/serviceModel/feed/atom";

        
        public ContentFeedHost(object singletonInstance, params Uri[] baseAddresses)
            : base(singletonInstance, baseAddresses)
        {
            

        }
        public ContentFeedHost(Type serviceType, params Uri[] baseAddresses)
            : base(serviceType, baseAddresses)
        {
        }
        public ServiceEndpoint  AddRssEndpoint(Type contract, string address)
        {
            return AddRssEndpoint(contract, address, new CustomBinding(new TextMessageEncodingBindingElement(MessageVersion.None, Encoding.UTF8), new HttpTransportBindingElement()));
        }
        public ServiceEndpoint AddRssEndpoint(Type contract, string address, Binding binding)
        {
            EndpointAddress epa = GetEndpointAddress(address, binding);
            if (epa == null)
            {
                throw new ArgumentException(
                    String.Format(("The specified address, {0}, is not an absolute address and the host has no base address matching the binding's scheme, {1}"), address, binding.Scheme));
            }
            //make a copy of ContractDescription
            //so that the user doesn't have trouble exposing SOAP endpoints from the same contract
            ContractDescription cd = ContractDescription.GetContract(contract);
            cd.Namespace = ContentFeedHost.RssContractNamespace;
            ServiceEndpoint endpoint = new ServiceEndpoint(cd, binding, epa);
            endpoint.Behaviors.Add(new MessageTransformBehavior(typeof(Rss20TransformProvider)));
            this.Description.Endpoints.Add(endpoint);
            this.ConfigureRawHttpEndpoint(endpoint);
            return endpoint;
        }

        public ServiceEndpoint AddAtomEndpoint(Type contract, string address)
        {
            return AddAtomEndpoint(contract, address, new CustomBinding(new TextMessageEncodingBindingElement(MessageVersion.None, Encoding.UTF8), new HttpTransportBindingElement() ));
        }
        public ServiceEndpoint AddAtomEndpoint(Type contract, string address, Binding binding)
        {
            EndpointAddress epa = GetEndpointAddress(address, binding);
            if (epa == null)
            {
                throw new ArgumentException(
                    String.Format(("The specified address, {0}, is not an absolute address and the host has no base address matching the binding's scheme, {1}"), address, binding.Scheme));
            }

            ContractDescription cd = ContractDescription.GetContract(contract);
            cd.Namespace = ContentFeedHost.AtomContractNamespace;
            ServiceEndpoint endpoint = new ServiceEndpoint(cd, binding, epa);
            endpoint.Behaviors.Add(new MessageTransformBehavior(typeof(Atom10TransformProvider)));
            this.Description.Endpoints.Add(endpoint);
            this.ConfigureRawHttpEndpoint(endpoint);
            return endpoint;
        }


        //public void EnableMetadata()
        //{
        //    ServiceMetadataBehavior smb = this.Description.Behaviors.Find<ServiceMetadataBehavior>();
        //    if (smb == null)
        //    {
        //        smb = new ServiceMetadataBehavior();
        //        this.Description.Behaviors.Add(smb);
        //    }
        //    smb.EnableGetWsdl = true;
        //    smb.EnableHelpPage = true;
        //    smb.EnableMetadataExchange = true;
        //}
        public void DisableMetadata()
        {
            ServiceMetadataBehavior smb = this.Description.Behaviors.Find<ServiceMetadataBehavior>();
            
            if (smb != null)
            {
                this.Description.Behaviors.Remove<ServiceMetadataBehavior>();
            }
        }
        void ConfigureRawHttpEndpoint(ServiceEndpoint endpoint)
        {
               endpoint.Behaviors.Add(new QueryStringFormatterBehavior());
               endpoint.Behaviors.Add(new UriDispatchBehavior());
               // HACK : this is just a workaround until we fix 54179
               endpoint.Contract.Behaviors.Add(HttpHelper.GetNoMetadataContractBehavior());
        }
        protected override void ApplyConfiguration()
        {
            base.ApplyConfiguration();
            this.DisableMetadata();
        }
        protected override void OnOpened()
        {
            base.OnOpened();
            foreach (ChannelDispatcher dispatcher in this.ChannelDispatchers)
            {
                TraceHelper.TraceEvent(TraceEventType.Information,"Endpoint listening at " + dispatcher.Listener.Uri,"ContentFeedHost.OnOpen");
                foreach (EndpointDispatcher endpoint in dispatcher.Endpoints)
                {
                    StringBuilder builder = new StringBuilder();
                    builder.Append("Endpoint address: ");
                    builder.Append(endpoint.EndpointAddress.ToString());
                    builder.Append("Endpoint contract filter: ");
                    builder.Append(endpoint.ContractFilter.ToString());
                    builder.Append("Endpoint address filter: ");
                    builder.Append(endpoint.AddressFilter.ToString());
                    TraceHelper.TraceEvent(TraceEventType.Information,builder.ToString(), "ContentFeedHost.OnOpen");
                }
            }
        }
        EndpointAddress GetEndpointAddress(string address, Binding binding)
        {
            EndpointAddress epa = null;
            if (!Uri.IsWellFormedUriString(address, UriKind.Absolute))
            {
                foreach (Uri baseaddr in this.BaseAddresses)
                {
                    if (baseaddr.Scheme == binding.Scheme)
                    {
                        string baseaddrstr=baseaddr.ToString();

                        if (!baseaddrstr.ToString().EndsWith("/") && !address.StartsWith("/"))
                        {
                            baseaddrstr += "/";
                        }
                        epa = new EndpointAddress(baseaddrstr + address);
                        break;
                    }
                }
            }
            else
            {
                epa = new EndpointAddress(address);
            }
            return epa;
        }
    
    }
}
