using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.ServiceModel.Channels;
using System.ServiceModel;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class HttpMessageContext
    {
        private HttpRequestMessageProperty requestProperty;
        private Uri requestUrl;
        private Dictionary<string,string> querystring;
        private HttpResponseMessageProperty responseProperty;
        private object lockobject;
        public HttpMessageContext(Message request)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }
            if (request.Headers.To != null)
            {
                this.requestUrl = request.Headers.To;
            }
            else if (request.Properties.ContainsKey("Via"))
            {
                this.requestUrl = request.Properties["Via"] as Uri;
            }
            
            if(request.Properties.ContainsKey(HttpRequestMessageProperty.Name))
            {
                this.requestProperty=request.Properties[HttpRequestMessageProperty.Name] as HttpRequestMessageProperty;
            }
            if (this.requestUrl != null && this.requestUrl.Query != null)
            {
                this.querystring = HttpHelper.ParseQueryString(this.requestUrl.Query);
            }
            else
            {
                this.querystring = new Dictionary<string,string>();
            }
            lockobject = new object();

        }
        public HttpRequestMessageProperty Request
        {
            get { return this.requestProperty; }
        }
  
        public Dictionary<string,string> QueryString
        {
            get { return this.querystring; }
        }
        public static HttpMessageContext SetCurrentContext(Message request)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }
            HttpMessageContext context = null;
            if (request.Properties.ContainsKey(HttpMessageContext.Name))
            {
                context = request.Properties[HttpMessageContext.Name] as HttpMessageContext;
            }
            else
            {
                context = new HttpMessageContext(request);
                request.Properties.Add(HttpMessageContext.Name, context);
            }
            return context;

        }
        public static HttpMessageContext Current
        {
            get 
            {
                if (OperationContext.Current != null && 
                    OperationContext.Current.RequestContext != null && 
                    OperationContext.Current.RequestContext.RequestMessage != null)
                {

                    Message request = OperationContext.Current.RequestContext.RequestMessage;
                    if (request.Properties.ContainsKey(HttpMessageContext.Name))
                    {
                        return request.Properties[HttpMessageContext.Name] as HttpMessageContext;
                    }
                    else if (request.Properties.ContainsKey(HttpRequestMessageProperty.Name))
                    {
                        HttpMessageContext context = new HttpMessageContext(request);
                        request.Properties.Add(HttpMessageContext.Name, context);
                        return context;
                    }
                    else
                    {
                        return null;
                    }
                }
                else
                {
                    return null;
                }
                
            }
        }
        public static string Name
        {
            get { return "HttpMessageContext"; }
        }
        public static HttpMessageContext GetContext(Message message)
        {
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }
            HttpMessageContext context=null;
            if (message.Properties.ContainsKey(HttpMessageContext.Name))
            {
                context = message.Properties[HttpMessageContext.Name] as HttpMessageContext;
            }
            else
            {
                context = HttpMessageContext.SetCurrentContext(message);
            }
            return context;
        }
    }
}
