using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.ServiceModel.Channels;

namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class PrefixAddressFilter : MessageFilter
    {
        Uri address;
        public PrefixAddressFilter(Uri address)
        {
            this.address = address;
        }
        public override bool Match(Message message)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "PrefixAddressFilter.Match");
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }
            bool matched = false;
            if (message.Headers.To != null)
            {
                matched= this.address.IsBaseOf(message.Headers.To);
            }
            else if (message.Properties.ContainsKey("Via"))
            {
                Uri via = message.Properties["Via"] as Uri;
                if (via != null)
                {
                    matched= this.address.IsBaseOf(via);
                }
            }
            TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, 
                String.Format((matched?"Message matched endpoint address {0}":"Message did not match endpoint address {0}"),this.address), 
                "PrefixAddressFilter.Match");
            return matched;

        }

        public override bool Match(MessageBuffer buffer)
        {
            throw new NotImplementedException();            
        }
    }
}
