//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.ServiceModel;
    using System.ServiceModel.Dispatcher;

    class UriPathSuffixOperationAddresser : IClientMessageInspector
    {
        Dictionary<string, string> actionDictionary;

        public UriPathSuffixOperationAddresser(Dictionary<string, string> actionDictionary)
        {
            if (actionDictionary == null)
                throw new ArgumentNullException("actionDictionary");

            this.actionDictionary = actionDictionary;
        }

        void IClientMessageInspector.AfterReceiveReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
        }

        object IClientMessageInspector.BeforeSendRequest(ref System.ServiceModel.Channels.Message request, IClientChannel channel)
        {
            string operationName;
            if (actionDictionary.TryGetValue(request.Headers.Action, out operationName))
            {
                request.Headers.Action = null;

                UriBuilder builder = new UriBuilder(request.Headers.To);
                builder.Path = builder.Path = "/" + operationName;
                request.Headers.To = builder.Uri;
                request.Properties.Via = builder.Uri;
            }
            return null;
        }
    }
}
