//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.Collections.Generic;
    using System.ServiceModel;
    using System.ServiceModel.Dispatcher;

    class UriPathSuffixOperationSelector : IDispatchOperationSelector
    {
        Uri endpointUri;
        Dictionary<string, string> operationNameDictionary;
        public UriPathSuffixOperationSelector(Uri endpointUri, Dictionary<string,string> operationNameDictionary)
        {
            if (operationNameDictionary == null)
                throw new ArgumentNullException("operationNameDictionary");

            if (endpointUri == null)
                throw new ArgumentNullException("endpointUri");

            if (endpointUri.ToString().EndsWith("/"))
            {
                this.endpointUri = new Uri(endpointUri.ToString().ToLower());
            }
            else
            {
                this.endpointUri = new Uri(endpointUri.ToString().ToLower() + "/");
            }

            
            this.operationNameDictionary = operationNameDictionary;
        }

        public string SelectOperation(ref System.ServiceModel.Channels.Message message)
        {
            TraceHelper.TraceMethod(System.Diagnostics.TraceEventType.Verbose, "UriOperationSelector.SelectOperation");
            Uri via = null;
            string realOperationName = null;
            if (message.Properties.ContainsKey("Via"))
            {
                via = message.Properties["Via"] as Uri;
            }
            HttpMessageContext.SetCurrentContext(message);

            if (via != null && !String.IsNullOrEmpty(via.ToString()))
            {
                Uri operationUri = new Uri(via.GetLeftPart(UriPartial.Path).ToString().ToLower());
                if (endpointUri.IsBaseOf(operationUri))
                {
                    string relUri = endpointUri.MakeRelativeUri(operationUri).ToString();
                    string operationName = null;
                    if (relUri.Contains("/"))
                    {
                        operationName = relUri.Substring(0, relUri.IndexOf('/'));
                    }
                    else
                    {
                        operationName = relUri;
                    }

                    if (!String.IsNullOrEmpty(operationName) && operationNameDictionary.ContainsKey(operationName))
                    {
                        realOperationName= operationNameDictionary[operationName];
                    }
                }
            }
            if (!String.IsNullOrEmpty(realOperationName))
            {
                TraceHelper.TraceEvent(System.Diagnostics.TraceEventType.Information, String.Format("Received request for operation named '{0}'", realOperationName), "UriOperationSelector.SelectOperation");
                return realOperationName;
            }
            else
            {
                Exception ex = new Exception(String.Format("Missing or unknown operation name in URL {0}", via));
                TraceHelper.WriteTraceData(System.Diagnostics.TraceEventType.Error, 0, "Unknown operation", ex.Message, "UriOperationSelector.SelectOperation", ex);
                throw ex;
            }
        }
    }
}
