<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:s="http://schemas.microsoft.com/2006/04/serviceModel/feed"
    xmlns:i="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:a="http://www.w3.org/2005/Atom"
   xmlns:date="http://exslt.org/dates-and-times"
   xmlns:xhtml="http://www.w3.org/1999/xhtml"
   exclude-result-prefixes="xsl i s date xhtml">
  <xsl:output omit-xml-declaration="no" method="xml" indent="yes"/>
  <xsl:template match="/">
    <feed xmlns="http://www.w3.org/2005/Atom" >
      <xsl:if test="string-length(normalize-space(//s:FeedLanguage/text())) > 0">
        <xsl:attribute name="xml:lang">
          <xsl:value-of select="normalize-space(//s:FeedLanguage/text())"/>
        </xsl:attribute> 
      </xsl:if>
      <xsl:apply-templates select="*"/>
    </feed>
    </xsl:template>


  <xsl:template match="s:FeedTitle">
    <a:title>
      <xsl:value-of select="normalize-space(text())"/>
    </a:title>
  </xsl:template>
  <xsl:template match="s:FeedWebPageLink">
    <a:link rel="alternate" type="text/html">
      <xsl:attribute name="href">
        <xsl:value-of select="normalize-space(text())"/>
      </xsl:attribute>
    </a:link>
  </xsl:template>
  <xsl:template match="s:FeedLink">
    <a:link rel="self" type="application/atom+xml">
      <xsl:attribute name="href">
        <xsl:value-of select="normalize-space(text())"/>
      </xsl:attribute>
    </a:link>
  </xsl:template>

  <xsl:template match="s:FeedDescription">
    <a:subtitle>
      <xsl:value-of select="normalize-space(text())"/>
    </a:subtitle>
  </xsl:template>
  <xsl:template match="s:FeedId">
    <a:id>
      <xsl:value-of select="normalize-space(text())"/>
    </a:id>
  </xsl:template>
    <xsl:template match="s:FeedLanguage">
    <!-- already processed as part of feed -->
  </xsl:template>
  <xsl:template match="s:FeedCopyright">
    <a:rights>
      <xsl:value-of select="text()"/>
    </a:rights>
  </xsl:template>  
  <xsl:template match="s:FeedLastUpdated">
    <a:updated>
      <xsl:value-of select="date:Rfc3339Date(normalize-space(text()))"/>
    </a:updated>
  </xsl:template>
  <xsl:template match="s:FeedCategories/s:Category">
    <a:category term="{s:CategoryValue}" label="{s:CategoryValue}" scheme="{s:CategoryDomain}"/>
  </xsl:template>
  <xsl:template match="s:FeedImage">
    <a:logo>
      <xsl:value-of select="text()"/>
    </a:logo>
  </xsl:template>
  <xsl:template match="s:FeedAuthor">
    <a:author>
      <a:name>
        <xsl:value-of select="normalize-space(text())"/>
      </a:name>
    </a:author>
  </xsl:template>
  <xsl:template match="s:FeedGenerator">
    <a:generator>
      <xsl:value-of select="text()"/>
    </a:generator>
  </xsl:template>
  <xsl:template match="s:FeedExtensions/*" >
    <xsl:copy-of select="."/>
  </xsl:template>
  <xsl:template match="s:FeedItems">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="s:FeedItem">
    <a:entry>
      <xsl:apply-templates select="*"/>
    </a:entry>
  </xsl:template>
  <xsl:template match="s:FeedItem/*" priority="-500">
    <xsl:copy-of select="."/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemId">
    <a:id>
      <xsl:value-of select="text()"/>
    </a:id>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemTitle">
    <a:title>
      <xsl:value-of select="normalize-space(text())"/>
    </a:title>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemWebPageLink">
    <a:link rel="alternate" type="text/html">
      <xsl:attribute name="href">
        <xsl:value-of select="normalize-space(text())"/>
      </xsl:attribute>
    </a:link>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemContent">
    <xsl:choose>
      <xsl:when test="./xhtml:div">
        <a:content type="xhtml">
          <xsl:copy-of select="./xhtml:div"/>
        </a:content>
      </xsl:when>
      <xsl:when test="./*">
        <a:content type="html">
          <xsl:copy-of select="./node()"/>
        </a:content>
      </xsl:when>
      <xsl:otherwise>
        <a:content type="html">
          <xsl:value-of select="text()"/>
        </a:content>
      </xsl:otherwise>
    </xsl:choose>

  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemAuthor">
    <a:author>
      <a:name>
        <xsl:value-of select="normalize-space(text())"/>
      </a:name>
    </a:author>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemPublishDate">
    <a:published>
      <xsl:value-of select="date:Rfc3339Date(normalize-space(text()))"/>
    </a:published>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemLastUpdated">
    <a:updated>
      <xsl:value-of select="date:Rfc3339Date(normalize-space(text()))"/>
    </a:updated>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemExtensions">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemExtensions/*">
    <xsl:copy>
      <xsl:value-of select="."/>
    </xsl:copy>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemCategories">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="s:FeedItem/s:ItemCategories/s:Category">
    <a:category term="{s:CategoryValue}" label="{s:CategoryValue}" scheme="{s:CategoryDomain}"/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemCommentsLink">
    <!-- suppress it -->
  </xsl:template>

  <xsl:template match="*[@i:nil='true']"/>
  <!-- suppress whitespace nodes -->
  <xsl:template match="text()" />
</xsl:stylesheet> 
