<?xml version="1.0" encoding="utf-8"?>

<xsl:stylesheet version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:s="http://schemas.microsoft.com/2006/04/serviceModel/feed"
    xmlns:i="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:date="http://exslt.org/dates-and-times"
     exclude-result-prefixes="s i date">
  <xsl:output 
    omit-xml-declaration="no" 
    method="xml" indent="yes"/>

<xsl:template match="/"> 
  <rss version="2.0">
    <channel>
      <xsl:apply-templates select="*"/>
    </channel>
  </rss>
</xsl:template>
  <xsl:template match="s:FeedTitle">
    <title>
      <xsl:value-of select="text()"/>
    </title>
  </xsl:template>
  <xsl:template match="s:FeedWebPageLink">
    <link>
      <xsl:value-of select="text()"/>
    </link>
  </xsl:template>
  <xsl:template match="s:FeedDescription">
    <description>
      <xsl:value-of select="text()"/>
    </description>
  </xsl:template>
  <xsl:template match="s:FeedLanguage">
    <language>
      <xsl:value-of select="text()"/>
    </language>
  </xsl:template>
  <xsl:template match="s:FeedCopyright">
    <copyright>
      <xsl:value-of select="text()"/>
    </copyright>
  </xsl:template>
  <xsl:template match="s:FeedLastUpdated">
    <lastBuildDate>
      <xsl:value-of select="date:Rfc1123Date(text())"/>
    </lastBuildDate>
  </xsl:template>
  <xsl:template match="s:FeedCategories/s:Category">
    <category>
      <xsl:attribute name="domain">
        <xsl:value-of select="s:CategoryDomain"/>
      </xsl:attribute>
      <xsl:value-of select="s:CategoryValue"/>
    </category>  
  </xsl:template>  
  <xsl:template match="s:FeedImage">
    <image>
      <url>
        <xsl:value-of select="text()"/>
      </url>
      <title>
        <xsl:value-of select="//s:FeedTitle"/>
      </title>
      <link>
        <xsl:value-of select="//s:FeedLink"/>
      </link>
    </image>
  </xsl:template>
  <xsl:template match="s:FeedAuthor">
    <managingEditor>
      <xsl:value-of select="text()"/>
    </managingEditor>
  </xsl:template>
  <xsl:template match="s:FeedGenerator">
    <generator>
      <xsl:value-of select="text()"/>
    </generator>
  </xsl:template>
  <xsl:template match="s:FeedId">
    <!-- suppress it -->
  </xsl:template>
  <xsl:template match="s:FeedLink">
    <!-- suppress it -->
  </xsl:template>
  <xsl:template match="s:FeedExtensions/*" >
    <xsl:copy-of select="."/>
  </xsl:template>
  
  <xsl:template match="s:FeedItems">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="s:FeedItem">
    <item>
      <xsl:apply-templates/>
    </item>
  </xsl:template>
  <xsl:template match="s:FeedItem/*" priority="-500">
    <xsl:copy-of select="."/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemId">
    <guid>
      <xsl:value-of select="text()"/>
    </guid>    
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemTitle">
    <title>
      <xsl:value-of select="text()"/>
    </title>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemWebPageLink">
    <link>
      <xsl:value-of select="text()"/>
    </link>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemContent">
    <description>
      <xsl:choose>
        <xsl:when test="./*">
          <xsl:copy-of select="string(./node())"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="text()"/>
        </xsl:otherwise>
      </xsl:choose>
    </description>
  </xsl:template>
<xsl:template match="s:FeedItem/s:ItemAuthor">
  <author>
    <xsl:value-of select="text()"/>
  </author>
</xsl:template>
<xsl:template match="s:FeedItem/s:ItemPublishDate">
  <pubDate>
      <xsl:value-of select="date:Rfc1123Date(text())"/>
  </pubDate>
</xsl:template>
<xsl:template match="s:FeedItem/s:ItemCommentsLink">
    <comments>
      <xsl:value-of select="text()"/>
    </comments>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemCategories">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemExtensions">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemCategories/s:Category">
    <category>
      <xsl:attribute name="domain">
        <xsl:value-of select="s:CategoryDomain"/>
      </xsl:attribute>
      <xsl:value-of select="s:CategoryValue"/>
    </category>
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemLastUpdated">
    <!-- suppress it -->
  </xsl:template>
  <xsl:template match="s:FeedItem/s:ItemExtensions/*">
    <xsl:copy-of select="."/>
  </xsl:template>

  <xsl:template match="*[@i:nil='true']"/>
<!-- suppress whitespace nodes -->
<xsl:template match="text()" />

</xsl:stylesheet> 
