#region Copyright Notice
// ----------------------------------------------------------------------------
// Copyright (C) 2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

// Author: VipulM
#endregion

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.IO;
    using System.Xml;
    using System.Text;
    using System.ServiceModel;
    using System.ServiceModel.Channels;
    using System.Diagnostics;
    public class PoxMessageEncoder : MessageEncoder
    {
        private PoxMessageEncoderFactory factory;
        private XmlWriterSettings writerSettings;
        private Encoding writeEncoding;

        internal PoxMessageEncoder(PoxMessageEncoderFactory factory)
        {
            this.factory = factory;
            this.writeEncoding = Encoding.GetEncoding(
                    this.factory.WriteEncoding);

            this.writerSettings = new XmlWriterSettings();
            writerSettings.Encoding = this.writeEncoding;
            writerSettings.OmitXmlDeclaration = false;
        }

        public override string ContentType
        {
            get 
            {
                return factory.WriteContentType;
            }
        }

        public override string MediaType
        {
            get 
            {
                return factory.WriteContentType;
            }
        }

        public override MessageVersion MessageVersion
        {
            get 
            {
                return factory.MessageVersion;
            }
        }

        public override bool IsContentTypeSupported(string contentType)
        {
            if (string.IsNullOrEmpty(this.factory.ReadContentType))
                return true;
            else
                return (string.Compare(this.factory.ReadContentType, 
                            contentType, true) == 0);
        }
        public override Message ReadMessage(ArraySegment<byte> buffer, BufferManager bufferManager, string contentType)
        {
            TraceHelper.TraceMethod(TraceEventType.Verbose, "PoxMessageEncoder.ReadMessage");
            Message msg = Message.CreateMessage(MessageVersion.Soap12WSAddressingAugust2004, "", XmlDictionaryReader.CreateTextReader(buffer.Array, XmlDictionaryReaderQuotas.Max));

            return msg;
        }
        public override Message ReadMessage(Stream stream, int maxSizeOfHeaders, string contentType)
        {
            TraceHelper.TraceMethod(TraceEventType.Verbose, "PoxMessageEncoder.ReadMessage");
            Message msg = Message.CreateMessage(MessageVersion.Soap12WSAddressingAugust2004, "", XmlDictionaryReader.CreateTextReader(stream, XmlDictionaryReaderQuotas.Max));

            return msg;
        }

        public override ArraySegment<byte> WriteMessage(Message message, 
                int maxMessageSize, BufferManager bufferManager, 
                int messageOffset)
        {
            TraceHelper.TraceMethod(TraceEventType.Verbose, "PoxMessageEncoder.WriteMessage");
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }
            if (bufferManager == null)
            {
                throw new ArgumentNullException("bufferManager");
            } 
            byte[] messageBytes = new byte[0];
            int messageLength = 0;

                // write it as XML message
                MemoryStream stream = new MemoryStream();

                XmlWriter writer = XmlWriter.Create(stream,this.writerSettings);
                XmlDictionaryWriter dictionaryWriter = 
                    XmlDictionaryWriter.CreateDictionaryWriter(writer);
                //This single line effectively removes the SOAP
                //envelope. Instead of telling the Message to 
                //write itself completely out (WriteMessage),
                //we tell it to only write out its body contents.
                //This skips writing out things like the SOAP envelope
                //and the SOAP header block.
                message.WriteBodyContents(dictionaryWriter);
                dictionaryWriter.Close();
                writer.Close();

                messageBytes = stream.GetBuffer();
                messageLength = (int)stream.Position;
                stream.Close();

            int totalLength = messageLength + messageOffset;
            byte[] totalBytes = bufferManager.TakeBuffer(totalLength);
            Array.Copy(messageBytes, 0, totalBytes, messageOffset, 
                    messageLength);

            ArraySegment<byte> byteArray = new ArraySegment<byte>(totalBytes, 
                    messageOffset, messageLength);

            return byteArray;
        }

        public override void WriteMessage(Message message, Stream stream)
        {
            TraceHelper.TraceMethod(TraceEventType.Verbose, "PoxMessageEncoder.WriteMessage");
            XmlWriter writer = XmlWriter.Create(stream, this.writerSettings);
            XmlDictionaryWriter dictionaryWriter = 
                XmlDictionaryWriter.CreateDictionaryWriter(writer);
            message.WriteBodyContents(dictionaryWriter);
            dictionaryWriter.Close();
            writer.Close();
        }
    }
}
