#region Copyright Notice
// ----------------------------------------------------------------------------
// Copyright (C) 2005 Microsoft Corporation, All rights reserved.
// ----------------------------------------------------------------------------

// Author: VipulM
#endregion

namespace Microsoft.ServiceModel.Samples.Syndication
{
    using System;
    using System.ServiceModel;
    using System.ServiceModel.Description;
    using System.ServiceModel.Channels;
    using System.ServiceModel.Security;
    using Microsoft.ServiceModel.Samples.Syndication.Configuration;
    //The binding element allows us to compose the POX message
    //encoding into the channel stack.
    public class PoxMessageEncodingBindingElement : MessageEncodingBindingElement
        
    {
        private string readContetType;
        private string writeContentType;
        private string writeEncoding;

        public PoxMessageEncodingBindingElement(string writeEncoding,
            string writeContentType, string readContentType)
        {
            this.WriteEncoding = writeEncoding;
            this.WriteContentType = writeContentType;
            this.ReadContentType = readContetType;
        }

        public PoxMessageEncodingBindingElement(string writeEncoding, 
            string writeContentType)
            : this(writeEncoding, writeContentType, 
                    ConfigurationStrings.DefaultReadContentType)
        {
            
        }

        public PoxMessageEncodingBindingElement(string writeEncoding)
            : this(writeEncoding, ConfigurationStrings.DefaultWriteContentType)
        {

        }

        public PoxMessageEncodingBindingElement()
            : this(ConfigurationStrings.DefaultWriteEncoding)
        {
        }

        public string WriteContentType
        {
            get 
            {
                return this.writeContentType;
            }
            
            set
            {
                this.writeContentType = value;
            }
        }


        public string ReadContentType
        {
            get
            {
                return this.readContetType;
            }

            set
            {
                this.readContetType = value;
            }
        }

        public string WriteEncoding
        {
            get
            {
                return this.writeEncoding;
            }

            set
            {
                this.writeEncoding = value;
            }
        }
    
        #region MessageEncodingBindingElement Members

        public override MessageVersion MessageVersion
        {
            get
            {
                return MessageVersion.CreateVersion(EnvelopeVersion.Soap12, AddressingVersion.WSAddressingAugust2004);
            }
            set
            {
                //ignore
            }
        }

        public override MessageEncoderFactory  CreateMessageEncoderFactory()
        {
            
            return new PoxMessageEncoderFactory(this.WriteEncoding,
                this.WriteContentType, this.ReadContentType);
        }

        #endregion
        
        
        public override BindingElement Clone()
        {
            return new PoxMessageEncodingBindingElement(this.WriteEncoding,
                this.WriteContentType, this.ReadContentType);
        }

        public override IChannelFactory<TChannel> BuildChannelFactory<TChannel>(BindingContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            //This line is critical! The transport will 
            //find its encoder by looking in Context.UnhandledBindingElements.
            //If you forget this line, you'll end up with the default
            //encoder for the transport even if this element is in the stack.

            //PB UnhandledBindingElements have been remove
//            context.UnhandledBindingElements.Add(this);
            context.BindingParameters.Add(this);
            return context.BuildInnerChannelFactory<TChannel>();
        }

        public override IChannelListener<TChannel> BuildChannelListener<TChannel>(BindingContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");
            //PB UnhandledBindingElements have been remove
            //            context.UnhandledBindingElements.Add(this);

            context.BindingParameters.Add(this);
            return context.BuildInnerChannelListener<TChannel>();

        }
        //public override T GetProperty<T>(BindingContext context)
        //{
        //    return default(T);
        //}

        public MessageVersion GetProperty(BindingContext context)
        {
            return this.MessageVersion;
        }

        //public override ChannelProtectionRequirements GetProtectionRequirements()
        //{
        //    return null;
        //}
    }
}
