using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.IO;
using System;
using System.Reflection;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class MessageTransform : IDispatchMessageInspector
    {
        IFormatTransformProvider transformer;
        public MessageTransform(Type transformProvider)
        {
            if (transformProvider == null)
            {
                throw new ArgumentNullException("transformProvider");
            }
            transformer = Activator.CreateInstance(transformProvider) as IFormatTransformProvider;

        }
        #region IDispatchMessageInspector Members
        public object AfterReceiveRequest(ref System.ServiceModel.Channels.Message request, IClientChannel channel, InstanceContext instanceContext)
        {
            if (request == null)
            {
                throw new ArgumentNullException("request");
            }

            if (!request.Properties.ContainsKey(HttpMessageContext.Name) && 
                request.Properties[HttpRequestMessageProperty.Name] != null)
            {
                HttpMessageContext.SetCurrentContext(request);
            }
            
            return null; //we don't need correlation
        }

        public void BeforeSendReply(ref System.ServiceModel.Channels.Message reply, object correlationState)
        {
            if (reply == null)
            {
                throw new ArgumentNullException("reply");
            }
            if (!reply.IsFault)
            {
                reply = TransformMessage(reply);
            }

        }


        #endregion
        private Message TransformMessage(Message message)
        {
            Message outmessage;
            if (message.IsEmpty)
            {

                outmessage=Message.CreateMessage(message.Version, message.Headers.Action);
            }
            else
            {
                MemoryStream stream = new MemoryStream();
                XmlDictionaryWriter writer = XmlDictionaryWriter.CreateTextWriter(stream);
                message.WriteBodyContents(writer);
                writer.Flush();
                stream.Position = 0;
                XmlDictionaryReader reader = XmlDictionaryReader.CreateTextReader(stream, XmlDictionaryReaderQuotas.Max);
                XmlReader outreader = transformer.Transform(reader);
                outmessage = Message.CreateMessage(message.Version, message.Headers.Action, outreader);
            }
            CopyHeadersAndProperties(message, outmessage);
            HttpResponseMessageProperty response=new HttpResponseMessageProperty();
            response.Headers.Add("Content-Type", transformer.ContentType);
            outmessage.Properties.Add(HttpResponseMessageProperty.Name, response);
            return outmessage;
        }

        private void CopyHeadersAndProperties(Message source, Message copy)
        {
            if (source.Version != copy.Version)
            {
                throw new InvalidOperationException(
                    String.Format("The source message version, {0}, does not match the copy message version, {1}",
                    source.Version,
                    copy.Version));
            }
            copy.Headers.Clear();
            copy.Headers.CopyHeadersFrom(source);
            copy.Properties.Clear();
            foreach(string key in source.Properties.Keys)
            {
                copy.Properties.Add(key, source.Properties[key]);
            }
        }
        private Message ExtractBody(Message message)
        {
            Message outmessage = Message.CreateMessage(MessageVersion.Soap12WSAddressingAugust2004, message.Headers.Action, message.GetReaderAtBodyContents());
            return outmessage;
        }


    }
}
