using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Xml.Xsl;
using System.Xml;
using System.Diagnostics;
namespace Microsoft.ServiceModel.Samples.Syndication
{
    public class XslTransformProvider : IFormatTransformProvider
    {

        XslCompiledTransform cachedXslt;
        XsltArgumentList xslArg;
        string contentType;

        public XslTransformProvider(XslCompiledTransform transform, string contentType)
        {
            this.contentType = contentType;
            InitializeTransform(transform);
        }

        internal XslTransformProvider(string transformName, string contentType) 
        {
            this.contentType = contentType;
            XslCompiledTransform transform=CacheXslTransform(transformName);
            InitializeTransform(transform);
            TraceHelper.TraceEvent(TraceEventType.Verbose,"Cached XslTransform named " + transformName,"XslTransformProvider");
            
        }
        private void InitializeTransform(XslCompiledTransform transform)
        {
            this.cachedXslt = transform;
            this.xslArg = new XsltArgumentList();
            xslArg.AddExtensionObject("http://exslt.org/dates-and-times", new ExsltDatesAndTimes()); 

        }
        private XslCompiledTransform CacheXslTransform(string transformName)
        {
            XmlTextReader reader = new XmlTextReader(new StringReader(RssResources.ResourceManager.GetString(transformName)));
            XslCompiledTransform xslt = new XslCompiledTransform();
            xslt.Load(reader);
            return xslt;
        }


        #region IFormatTransformProvider Members

        public System.Xml.XmlReader Transform(System.Xml.XmlReader input)
        {
            TraceHelper.TraceEvent(TraceEventType.Verbose, "Transforming message", "XslTransformProvider.Transform");
            MemoryStream stream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(stream, new UTF8Encoding());
            cachedXslt.Transform(input,xslArg, writer);
            stream.Position = 0;
            XmlTextReader reader = new XmlTextReader(stream);
            return reader;
        }
        public string ContentType
        {
            get { return this.contentType; }
        }

        #endregion

    }
}
